import { kebabToPascal } from "../case.js";
import { getSearchParamsTypeCode, getTailRuleForm, urichkPathToString, } from "./nextjs-navigation-hook.js";
export function compile(schema, config) {
    const { name = "" } = config ?? {};
    const defaultFunctionName = `get${kebabToPascal(name)}Path`;
    return `
import {encode} from 'querystring';

export interface PathParamsTable {${schema.map((rule) => {
        const path = JSON.stringify(urichkPathToString(rule.head.path || []));
        const tailRuleForm = getTailRuleForm(rule.tail);
        const searchParamsType = getSearchParamsTypeCode(tailRuleForm);
        return `
        ${rule.comment ?? ""}
        [${path}]: ${searchParamsType};
      `;
    }).join("\n")}
}

export default function ${defaultFunctionName}<Path extends keyof PathParamsTable>(path: Path, searchParams?: PathParamsTable[Path]): string {
  const query = (
      searchParams ?
      '?' + encode(searchParams) :
      ''
    );
  return path + query;
}
  `;
}
