"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.compile = void 0;
const case_js_1 = require("../case.js");
const nextjs_navigation_hook_js_1 = require("./nextjs-navigation-hook.js");
function compile(schema, config) {
    const { name = "" } = config ?? {};
    const defaultFunctionName = `get${(0, case_js_1.kebabToPascal)(name)}Path`;
    return `
import {encode} from 'querystring';

export interface PathParamsTable {${schema.map((rule) => {
        const path = JSON.stringify((0, nextjs_navigation_hook_js_1.urichkPathToString)(rule.head.path || []));
        const tailRuleForm = (0, nextjs_navigation_hook_js_1.getTailRuleForm)(rule.tail);
        const searchParamsType = (0, nextjs_navigation_hook_js_1.getSearchParamsTypeCode)(tailRuleForm);
        return `
        ${rule.comment ?? ""}
        [${path}]: ${searchParamsType};
      `;
    }).join("\n")}
}

export default function ${defaultFunctionName}<Path extends keyof PathParamsTable>(path: Path, searchParams?: PathParamsTable[Path]): string {
  const query = (
      searchParams ?
      '?' + encode(searchParams) :
      ''
    );
  return path + query;
}
  `;
}
exports.compile = compile;
