"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const index_js_1 = require("../visitor/index.js");
const printer_js_1 = require("./printer.js");
function format(ast) {
    const printer = (0, printer_js_1.createPrinter)();
    const formatter = {
        ...index_js_1.visitor,
        visitToken(visitor, node) {
            printer.print(node.text);
        },
        visitHead(visitor, node) {
            index_js_1.visitor.visitHead(visitor, node);
            printer.print(" ");
        },
        visitUserinfo(visitor, node) {
            index_js_1.visitor.visitUserinfo(visitor, node);
            printer.print("@");
        },
        visitScheme(visitor, node) {
            index_js_1.visitor.visitScheme(visitor, node);
            printer.print(":");
        },
        visitPort(visitor, node) {
            printer.print(":");
            index_js_1.visitor.visitPort(visitor, node);
        },
        visitPathFragment(visitor, node) {
            printer.print("/");
            index_js_1.visitor.visitPathFragment(visitor, node);
        },
        visitParamPathFragment(visitor, node) {
            printer.print("[");
            index_js_1.visitor.visitParamPathFragment(visitor, node);
            printer.print("]");
        },
        visitTail(visitor, node) {
            (0, index_js_1.visitStatementBase)(visitor, node, () => {
                printer.print("{");
                printer.indent();
                for (const rule of node.rules) {
                    printer.print("\n");
                    visitor.visitTailRule(visitor, rule);
                    printer.print("\n");
                }
                printer.dedent();
                printer.println("}\n");
            });
        },
        visitTailRule(visitor, node) {
            index_js_1.visitor.visitTailRule(visitor, node);
        },
        visitTailRuleMatch(visitor, node) {
            (0, index_js_1.visitStatementBase)(visitor, node, () => {
                printer.printIndent();
                const { tailType, matchType, pattern } = node;
                visitor.visitTailType(visitor, tailType);
                visitor.visitMatchType(visitor, matchType);
                if (pattern.length > 1) {
                    printer.indent();
                    for (const value of pattern) {
                        printer.print("\n");
                        printer.printIndent();
                        printer.print("| ");
                        visitor.visitTailRulePatternValue(visitor, value);
                    }
                    printer.dedent();
                }
                else {
                    for (const value of pattern) {
                        visitor.visitTailRulePatternValue(visitor, value);
                    }
                }
            });
        },
        visitTailRuleForm(visitor, node) {
            (0, index_js_1.visitStatementBase)(visitor, node, () => {
                printer.printIndent();
                const { tailType, matchType, pattern } = node;
                visitor.visitTailType(visitor, tailType);
                visitor.visitMatchType(visitor, matchType);
                printer.print("{\n");
                printer.indent();
                for (const rule of pattern) {
                    visitor.visitTailRuleFormPatternRule(visitor, rule);
                }
                printer.dedent();
                printer.printIndent();
                printer.print("}");
            });
        },
        visitTailRuleFormPatternRule(visitor, node) {
            (0, index_js_1.visitStatementBase)(visitor, node, () => {
                printer.printIndent();
                const { key, value, array } = node;
                visitor.visitKey(visitor, key);
                if (array)
                    visitor.visitArrayToken(visitor, array);
                if (value) {
                    printer.print(" ");
                    printer.print("=");
                    printer.print(" ");
                    value.forEach((patternValue, index) => {
                        if (index > 0)
                            printer.print(" | ");
                        visitor.visitTailRulePatternValue(visitor, patternValue);
                    });
                }
                printer.print("\n");
            });
        },
        visitTailRulePatternValue(visitor, node) {
            index_js_1.visitor.visitTailRulePatternValue(visitor, node);
        },
        visitArrayToken(visitor, node) {
            printer.print(" ");
            visitor.visitToken(visitor, node);
        },
        visitTailType(visitor, node) {
            index_js_1.visitor.visitTailType(visitor, node);
            printer.print(" ");
        },
        visitMatchType(visitor, node) {
            index_js_1.visitor.visitMatchType(visitor, node);
            printer.print(" ");
        },
        visitCommentGroup(visitor, node) {
            printer.printIndent();
            for (const comment of node.comments) {
                visitor.visitComment(visitor, comment);
                printer.print("\n");
            }
        },
        visitComment(visitor, node) {
            if (node.type === "document-comment") {
                node.text.split("\n").map((line, index) => {
                    const trimed = line.trimStart();
                    if (index) {
                        printer.println("");
                        printer.printIndent();
                        printer.print(" ");
                    }
                    printer.print(trimed);
                });
            }
            else {
                visitor.visitToken(visitor, node);
            }
        },
    };
    formatter.visitUrichk(formatter, ast);
    return printer.done();
}
exports.default = format;
